#include<pthread.h>

typedef struct{
    int counter;
    pthread_mutex_t mutex;
    pthread_cond_t cond;
}sem_t;

int sem_init(sem_t* sem){
    if(pthread_mutex_init(&sem->mutex,NULL))
        return -1;
    if(pthread_cond_init(&sem->cond,NULL)){
        pthread_mutex_destroy(&sem->mutex);
        return -1;
    }
    sem->counter=0;
    return 0;
}

int sem_wait(sem_t* sem){
    if(pthread_mutex_lock(&sem->mutex))
        return -1;
    if(sem->counter==0){
        if(pthread_cond_wait(&sem->cond,&sem->mutex)){
            pthread_mutex_unlock(&sem->mutex);
            return -1;
        }
    }
    sem->counter--;
    if(pthread_mutex_unlock(&sem->mutex))
        return -1;
    return 0;
}

int sem_signal(sem_t*sem){
    if(pthread_mutex_lock(&sem->mutex))
        return -1;
    sem->counter++;
   if(pthread_cond_signal(&sem->cond)){
        pthread_mutex_unlock(&sem->mutex);
        return -1;
    }
    if(pthread_mutex_unlock(&sem->mutex))
        return -1;
    return 0;
}

int sem_destroy(sem_t*sem){
    if(pthread_cond_destroy(&sem->cond))
        return -1;
    if(pthread_mutex_destroy(&sem->mutex))
        return -1;
    return 0;
}
