/* POSIX_barrier.c - implementacja bariery */
#include "POSIX_barrier.h"

/* barrier_init - inicjalizacja zmiennej barierowej */
barrier_t barrier_init(int max)
{
  barrier_t barrier;
  barrier = (barrier_t) malloc(sizeof(struct barrier_struct));
  if (barrier != NULL) {
    barrier->maxcnt = max;
    barrier->cnt    = 0; barrier->phase  = 0;
    pthread_mutex_init(&barrier->lock, NULL);
    pthread_cond_init(&barrier->cond, NULL);
  }
  return(barrier);
}

/* barrier_wait - czekanie przed bariera az przyjdzie maxcnt watkow */
barrier_wait(barrier_t barrier)
{
  int my_phase;
  pthread_mutex_lock(&barrier->lock);
  my_phase = barrier->phase;
  barrier->cnt++;
  if (barrier->cnt == barrier->maxcnt) { /* Jestem ostatni */
    barrier->cnt = 0;            /* zresetuj przed nastepnym uzyciem */
    barrier->phase = 1 - my_phase;
    pthread_cond_broadcast(&barrier->cond);
  }
  while (barrier->phase == my_phase) {
            /* 'while' a nie 'if' z powodu mozliwosci */
            /* falszywych budzen */
    pthread_cond_wait(&barrier->cond, &barrier->lock);
  }
  pthread_mutex_unlock(&barrier->lock);
}

/* barrier_destroy - zniszczenie zmiennej barierowej */
void barrier_destroy(barrier_t barrier)
{
   pthread_mutex_destroy(&(barrier->lock));
   pthread_cond_destroy(&(barrier->cond));
   free(barrier);
}