/* url_gj_omp.c
 * Rozwiazywanie ukladow rownan liniowych metoda eliminacji
 * Gaussa-Jordana
 * @author Piotr Zarowny, Politechnika Warszawska
 */
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <omp.h>

double** A = NULL;                 // macierz A
double* b = NULL;                  // wektor b
double* x = NULL;                  // wektor x
int n;                             // wymiar macierzy/wektorw

FILE *fx;                          // plik wynikw

int i, j, k = 0, l;                // zmienne pomocnicze
double alfa;

int main(int argc, char* argv[]) {
                                   // pobranie wymiaru zadania n
    if (argc == 2) {
        sscanf(argv[1], "%d", &n);
    } else {
        printf("\nProsze podac wymiar n: ");
        scanf("%d", &n);
    }
    if (n <= 0) {
        printf("\nBLAD: podano bledna wartosc wymiaru n!\n");
        exit(1);
    }
    printf("\nObliczanie... ");
                                   // utworzenie A, b, x
    if ((A = (double **)malloc((n + 1) * sizeof(double *))) == NULL) {
        printf("\nBLAD: nie utworzono macierzy A!\n");
        exit(1);
    }
    for (i = 0; i <= n; i++)
        A[i] = (double *)malloc((n + 1) * sizeof(double));

    if ((b = (double *)malloc((n + 1) * sizeof(double))) == NULL) {
        printf("\nBLAD: nie utworzono wektora b!\n");
        exit(1);
    }
    if ((x = (double *)malloc((n + 1) * sizeof(double))) == NULL) {
        printf("\nBLAD: nie utworzono wektora x!\n");
        exit(1);
    }
                                   // wypenienie A, b, x
    for (i = 1; i <= n; i++) {
        for (j = 1; j <= n; j++)
           A[j][i] = (i != j) ? pow(-1, i + j - 1) / (i + j - 1) : 2 +
                                                 pow(-1, i) / pow(i, 2);

        b[i] = 3 + pow(-1, i + 1) * sin(i);
        x[i] = 0;
    }

    // algorytm eliminacji Gaussa-Jordana
    for (k = 1; k <= n; k++) {

    #pragma omp parallel for private(alfa, j)
        for (i = 1; i <= n; i++) {
            if (i == k)
	        continue;

            alfa = A[i][k] / A[k][k];

            for (j = k; j <= n; j++)
                A[i][j] = A[i][j] - alfa * A[k][j];

            b[i] = b[i] - alfa * b[k];
        }
    }

    for (l = 1; l <= n; l++)
        x[l] = b[l] / A[l][l];
                                   // zapis wynikw do pliku
    if ((fx = fopen("wyniki.txt", "w")) != NULL) {
        for(l = 1; l < = n; l++)
           fprintf(fx, "%.4f\n",x[l]);

        fclose(fx);
    } else
        printf("\nBLAD: nie zapisano wynikow do pliku!\n");
                                   // czyszczenie pamici
    for(l = 0; l <= n; l++)
        free(A[l]);
    free(A);
    free(b);
    free(x);
                                   // zakoczenie programu
    printf("OK\n\n");

    return 0;
}

