/* int_liczba_pi.c
 * Program do szacowania liczby pi.
 * Parametr podawany do programu - N: liczba przedzialow.
*/
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "mpi.h"
#define fun(x) ((float)(4.0/(1.0+x*x)))
#define pi ((float)(4.0*atan(1.0)))

MPI_Status status;
main(int argc, char **argv) {
    float   blad, sum, d, sum_cz;
    int     k, i, N, info, procId, nprocs, type = 2;

// Wszystkie instancje programu pobieraja swoj indeks procesu (procId)
// i liczbe procesow w grupie (nprocs)
    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &procId);
    MPI_Comm_size(MPI_COMM_WORLD, &nprocs);

// Pobranie przez proces glowny parametru "N"
// (liczba przedzialow do aproksymacji)
    if (procId == 0) {
        printf ("Obliczanie liczby PI \n");
        printf( "Podaj liczbe przedzialow do aproksymacji(0 - WYJSCIE): ");
        scanf ("%d", &N);
    }
    MPI_Bcast(&N, 1, MPI_INT, 0, MPI_COMM_WORLD);

// Sprawdzenie warunku zakonczenia
    if (N <= 0) {
        printf("wezel %d - koniec\n", procId);
        exit(0);
    }

    while (N > 0) {

// Wykonanie obliczen w N krokach przez "nprocs" instancji programu
// Kazdy proces wykonuje 1/nprocs obliczen, zaczynajac od procId+1
// poniewaz procId = 0, 1, 2...,
        d = 1.0 / (float)N;
        sum = 0.0;
        for (k = procId + 1; k <= N; k += nprocs)
            sum = sum + fun(((float)k - 0.5) * d);
        sum  = sum * d;
        blad = sum - pi;

// Zebranie przez proces glowny wynikow od wszystkich procesow
// i wyswietlenie wyniku koncowego (sum)
        if (procId == 0) {
            printf ("proces glowny obliczyl sum_cz = %7.5f\n", sum);
            for (i = 1; i < nprocs; i++) {
                info = MPI_Recv(&sum_cz, 1, MPI_FLOAT, i, type,
                                          MPI_COMM_WORLD, &status);
                printf ("proces %d obliczyl sum_cz = %7.5f\n", i, sum_cz);
                sum=sum+sum_cz;
           }
           blad = sum - pi;
           printf ("sum = %7.5f, blad = %10e\n", sum, blad);
           fflush(stdout);
        }

// Pozostale procesy wysylaja swoje wyniki (sum_cz) do procesu
// glownego i czekaja na nastepny zestaw danych do obliczen
        else {
             info = MPI_Send(&sum, 1, MPI_FLOAT, 0, type,
                                                    MPI_COMM_WORLD);
             if (info != 0) {
                 printf ("numer procesu, %d blad wysylania\n", procId);
                 exit(0);
             }
             printf ("proces %d wyslal wyniki: %7.5f do procesu glownego\n",
                                                               procId, sum);
             fflush(stdout);
         }
// Pobranie nowej liczby przedzialow do aproksymacji
        if (procId == 0) {
            printf ("Podaj nowa liczbe przedzialow do aproksymacji lub \
                                                          '0' - KONIEC:\n");
            scanf ("%d", &N);
        }
        MPI_Bcast(&N, 1, MPI_INT, 0, MPI_COMM_WORLD);
    }
    MPI_Finalize();
    return 0;
}

