import java.rmi.*;			

class SideBarrier {
	int stoppedCount;		

	SideBarrier() {
		stoppedCount = 0;	
	}
	public synchronized void waitForTheOthers() throws Exception {
		stoppedCount++;
		if( stoppedCount == 2 ) {
			stoppedCount = 0;
			notifyAll();	
		}
		else
			wait();			
	}
}
class SideThread extends Thread {
	int         iStart;				
	int         iEnd;				
	float       snapshot[];			
	String      hostName;			
	SideBarrier barrier;			
	SideThread( String      hostName,
				int         iStart,
				int         iEnd,
				float       snapshot[],
				SideBarrier barrier) {
		this.iStart   = iStart;
		this.iEnd     = iEnd;
		this.snapshot = snapshot;
		this.hostName = hostName;
		this.barrier  = barrier;
	}
	public void run() {
		try {
			MyIterationInterface obj = ( MyIterationInterface )
					Naming.lookup( "//" + hostName + "/MyIteration" );
			float x[] = new float[ snapshot.length ];	
			MyIterationParameters parameters =
											new MyIterationParameters();
			parameters.x = x;
			parameters.iStart = iStart;		
			parameters.iEnd   = iEnd;		
			while( true ) {					
				synchronized ( snapshot ) {	
					System.arraycopy( snapshot, 0, parameters.x, 0,
														x.length );
				}
				parameters = obj.MyIteration( parameters );	
				synchronized ( snapshot ) {	
					System.out.print( "Thread " + hostName + ":" );
					for( int i = 0; i < parameters.x.length; i++ ) {
						System.out.print( " " + parameters.x[ i ] );
						if( i >= iStart && i <= iEnd )
							snapshot[ i ] = parameters.x[ i ];
					}
					System.out.println();
				}
				barrier.waitForTheOthers();
			}
		}
		catch( Exception e ) {
			e.printStackTrace();
			return;
		}
	}
}
public class example4client {
	public static void main( String arg[] ) throws Exception {
		if( arg.length < 2 ) {					
			System.out.println(
						"specify hostname1 and hostname2" );
			return;

		}
		final int DIM = 5;						
		float snapshot[] = new float[ DIM ];	
		for( int i = 0; i < DIM; i++ )			
			snapshot[ i ] = i + 1;
		SideBarrier barrier = new SideBarrier();	
		Thread thread1 = new SideThread( arg[0], 0,     1, snapshot,
															barrier );
		thread1.start();
		Thread thread2 = new SideThread( arg[1], 2, DIM-1, snapshot,
															barrier );
		thread2.start();
		thread1.join();	
		thread2.join();
	}
}
