#include <OB/CORBA.h>
#include <OB/CosNaming.h>
#include <stdio.h>
#include "MyIterationInterface.h"
#define DIM 5

float      Snapshot[ DIM ];
JTCMutex   SnapshotMutex;
JTCMonitor BarrierMonitor;

class SideThread: public JTCThread
{
	char         *pName;
	int           iStart;
	int           iEnd;
	CORBA_ORB_var vOrb;
	static int    stoppedCount;
	void waitForTheOthers()
	{
		JTCSynchronized sync( BarrierMonitor );
		stoppedCount++;
		if( stoppedCount == 2 )
        {
			stoppedCount = 0;
			BarrierMonitor.notifyAll();
		}
		else
			BarrierMonitor.wait();
	}
public:
	SideThread( char *pName, int iStart, int iEnd, CORBA_ORB_var vOrb )
    :   pName( pName ), iStart( iStart ), iEnd( iEnd), vOrb( vOrb ) {}
	virtual void run()
	{
		CORBA_Object_var vObject =
                vOrb->resolve_initial_references( "NameService" );
		CosNaming_NamingContext_var vNamingContext =
				CosNaming_NamingContext::_narrow( vObject );
		CosNaming_Name implName;
		implName.length( 1 );
		implName[ 0 ].id   = CORBA_string_dup( pName );
		implName[ 0 ].kind = CORBA_string_dup( "" );
		CORBA_Object_var vCorbaObj =
                vNamingContext->resolve( implName );
		MyIterationInterface_var vMyIteration =
				MyIterationInterface::_narrow( vCorbaObj );
		float tmpSnapshot[ DIM ];
		myVector snapshot( DIM, DIM, tmpSnapshot );
		while( !0 )
        {
			SnapshotMutex.lock();
			for( int i = 0; i < DIM; i++ )
				snapshot[ i ] = Snapshot[ i ];
			SnapshotMutex.unlock();
			vMyIteration->myIteration( snapshot, iStart, iEnd );
			SnapshotMutex.lock();
			for( i = 0;
                 i < DIM;
                 i++ )
            {
				printf( " %f", snapshot[ i ] );
			}
			printf( "\n" );
			SnapshotMutex.unlock();
			waitForTheOthers();
		}
	}
};

int SideThread::stoppedCount = 0;
int main( int argc, char *argv[] )
{
	JTCInitialize initialize;
	for( int i = 0; i < DIM; i++ )
		Snapshot[ i ] = i + 1;
	if( argc < 3 )
    {
		printf( "Usage: %s 1st_name 2nd_name -ORBnaming ior\n", argv[ 0 ] );
		return 1;
	}
	CORBA_ORB_var vOrb;
	try {
		vOrb = CORBA_ORB_init( argc, argv );
	}
	catch( CORBA_SystemException& e ) {
		return 1;
	}

    JTCThread *pThread1 = new SideThread( argv[ 1 ], 0,       1, vOrb );
	JTCThread *pThread2 = new SideThread( argv[ 2 ], 2, DIM - 1, vOrb );
	pThread1->start();
	pThread2->start();
	pThread1->join();
	pThread2->join();
	return 0;
}
