#include <OB/CORBA.h>
#include <OB/Util.h>
#include <OB/CosEventChannelAdmin.h>
#include <OB/CosEventComm_skel.h>
#include <stdio.h>
#include "MyIterationParameters.h"
#define DIM 5
extern int MyIteration( float *pSnapshot, int length, int iStart, int iEnd );
int      Istart;
int      Iend;
float    Snapshot[ DIM ];
JTCMutex SnapshotMutex;
class PushSupplierThread: public CosEventComm_PushSupplier_skel,
						  public JTCThread
{
	CosEventChannelAdmin_EventChannel_var vChannel;
	CosEventComm_PushConsumer_var         vConsumer;
public:
	PushSupplierThread(
                    CosEventChannelAdmin_EventChannel_var vChannel )
	:   vChannel( CosEventChannelAdmin_EventChannel::_duplicate(
                                                        vChannel ) ) {}
	~PushSupplierThread()
	{
		vConsumer->disconnect_push_consumer();
	}
	void disconnect_push_supplier()
    {
		vConsumer = CosEventComm_PushConsumer::_nil();
	}
	virtual void run()
	{
		try {
			CosEventChannelAdmin_SupplierAdmin_var
                            vSupplierAdmin = vChannel->for_suppliers();
			CosEventChannelAdmin_ProxyPushConsumer_var
                            vProxyConsumer =
									vSupplierAdmin->obtain_push_consumer();
			vProxyConsumer->connect_push_supplier(
									CosEventComm_PushSupplier::_nil() );
			vConsumer = CosEventComm_PushConsumer::_duplicate(
                                                        vProxyConsumer );
			float x[ DIM ];
			OBFixSeq< CORBA_Float > tmpSnapshot( DIM, DIM, x );
			struct MyIterationParameters myParameters;
			myParameters.iStart   = Istart;
			myParameters.iEnd     = Iend;
			myParameters.snapshot = tmpSnapshot;
			while( !0 )
            {
				SnapshotMutex.lock();
				memcpy( x, Snapshot, sizeof( x ) );
				SnapshotMutex.unlock();
				myParameters.errorCode =
                                    MyIteration( x, DIM, Istart, Iend );
				for( int i = 0; i < DIM; i++ )
                {
					myParameters.snapshot[ i ] = x[ i ];
					printf( "%f", x[ i ] );
				}
				printf( "\n" );
				fflush( stdout );
				CORBA_Any anyType;
				anyType <<= myParameters;
				vConsumer->push( anyType );
			}
		}
		catch( CORBA_SystemException& e ) {
			OBPrintException( e );
		}
	}
};
class PushConsumer: public CosEventComm_PushConsumer_skel
{
public:
	void push( const CORBA_Any& anyType )
	{
		struct MyIterationParameters *pMyParameters = NULL;
		if( anyType >>= pMyParameters )
        {
			SnapshotMutex.lock();
			for( int i = pMyParameters->iStart;
                                          i <= pMyParameters->iEnd; i++ )
				Snapshot[ i ] = pMyParameters->snapshot[ i ];
			printf( "{%ld-%ld} ", pMyParameters->iStart,
                                  pMyParameters->iEnd );
			fflush( stdout );
			SnapshotMutex.unlock();
			delete pMyParameters;
			pMyParameters = NULL;
		}
	}
	void disconnect_push_consumer() {}
};
int main( int argc, char *argv[] )
{
	JTCInitialize initialize;
	for( int i = 0; i < DIM; i++ )
		Snapshot[ i ] = i + 1;
	if( argc < 3 ) {
		printf( "Usage: %s startIndex endIndex\n", argv[ 0 ] );
		return 1;
	}
	if( !sscanf( argv[ 1 ], "%d", &Istart ) ||
		                            !sscanf( argv[ 2 ], "%d", &Iend   ) )
    {
		printf( "wrong arguments format\n" );
		return 1;
	}
	try {
		CORBA_ORB_var vOrb = CORBA_ORB_init( argc, argv );
		CORBA_Object_var vObject =
                vOrb->resolve_initial_references( "EventService" );
		CosEventChannelAdmin_EventChannel_var vEventChannel =
				CosEventChannelAdmin_EventChannel::_narrow( vObject );
		PushSupplierThread *pSupplier =
                new PushSupplierThread( vEventChannel );
		pSupplier->start();
		CORBA_BOA_var vBoa = vOrb->BOA_init( argc, argv );
		CosEventComm_PushConsumer_var vMyConsumer = new PushConsumer;
		CosEventChannelAdmin_ConsumerAdmin_var vConsumerAdmin =
				vEventChannel->for_consumers();
		CosEventChannelAdmin_ProxyPushSupplier_var vProxySupplier =
				vConsumerAdmin->obtain_push_supplier();
		vProxySupplier->connect_push_consumer( vMyConsumer );
		vBoa->impl_is_ready( CORBA_ImplementationDef::_nil() );
	}
	catch( CORBA_SystemException& e ) {
		OBPrintException( e );
	}
	return 0;
}
