//   Przyklad UPC - calkowanie numeryczne
//   Obliczanie liczby Pi
//      Zrodlo:
//         UPC Manual
//         The George Washington University
//             High Performance Computing Laboratory
//
#include<upc_relaxed.h>
#include<math.h>
#include<stdlib.h>
#define N 10000000L
#define f(x) (1.0 / (1.0 + x * x))

upc_lock_t *l;
shared double pi = 0.0;

int main(int argc, char** argv) {
    double local_pi = 0.0;
    long i;
    long NN = (argc > 1) ? atoi(argv[0]) : N;
    bupc_tick_t start, end;

    if(MYTHREAD == 0)
        start=bupc_ticks_now();
    l = upc_all_lock_alloc();
    upc_forall(i = 0; i < NN; i++; i)
        local_pi += (double) f((0.5 + i) / (NN));
    local_pi *= (double)(4.0 / NN);
    upc_lock(l);
       pi += local_pi;
    upc_unlock(l);
    upc_barrier;
    if(MYTHREAD == 0){
        end = bupc_ticks_now();
        printf("PI=%.16f\n",pi);
        double secs = bupc_ticks_to_us(end - start) / 1000000.0;
        printf("Time was: %d microseconds, %11.6f seconds\n",
                     (int)bupc_ticks_to_us(end - start), secs);
        upc_lock_free(l);
    }
}

